/**
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.component.framework;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.component.ComponentLifeCycle;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.management.ObjectName;

import org.ow2.petals.component.framework.api.Component;
import org.ow2.petals.component.framework.api.Interceptor;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.api.su.ServiceUnitExtensionsValidator;
import org.ow2.petals.component.framework.api.su.ServiceUnitListener;
import org.ow2.petals.component.framework.interceptor.description.InterceptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.CDKJBIDescriptorException;
import org.ow2.petals.component.framework.jbidescriptor.JBIDescriptorBuilder;
import org.ow2.petals.component.framework.jbidescriptor.generated.ComponentInterceptor;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Runtimeboolean;
import org.ow2.petals.component.framework.listener.AsyncMessageHandler;
import org.ow2.petals.component.framework.listener.JBIAcceptorManager;
import org.ow2.petals.component.framework.listener.JBIWorkerManager;
import org.ow2.petals.component.framework.listener.WorkerPoolManager;
import org.ow2.petals.component.framework.mbean.InterceptorManager;
import org.ow2.petals.component.framework.mbean.MBeanHelper;
import org.ow2.petals.component.framework.mbean.RuntimeConfigurationNotifier;
import org.ow2.petals.component.framework.performance.PerformanceNotification;
import org.ow2.petals.component.framework.performance.PerformanceNotifier;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.component.framework.util.StringHelper;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentFragment;

import static org.ow2.petals.component.framework.Constants.Component.JBI_XML;
import static org.ow2.petals.component.framework.Constants.Component.META_INF;

/**
 * An abstract class that can be used to easily create a new JBI Component. It
 * implements Component, ComponentLifeCycle and MessageExchangeProcessor JBI
 * interfaces.
 * 
 * @author ofabre - EBMWebsourcing
 * @since 1.0
 * 
 */
public abstract class AbstractComponent implements Component {

    /**
     * The component's delivery channel
     */
    private DeliveryChannel channel;

    /**
     * The Component's Context
     */
    protected ComponentContext context;

    /**
     * The component configuration
     */
    private Jbi jbiComponentConfiguration;

    /**
     * The component specific configuration in extensions form.
     */
    private ConfigurationExtensions componentExtensions;

    /**
     * The message exchange acceptor manager
     */
    private JBIAcceptorManager acceptorManager;

    /**
     * The component's logger
     */
    private Logger logger;

    /**
     * The ServiceUnit lifecycle listener
     */
    private ServiceUnitListener serviceUnitListener;

    /**
     * The component's service unit manager
     */
    private ServiceUnitManager suManager;

    /**
     * The ServiceUnit validator.
     * 
     * @deprecated Define constraints in the component JBI descriptor schema
     *  instead.
     */
    private ServiceUnitExtensionsValidator serviceUnitExtensionsValidator;

    /**
     * The Worker manager in charge of handling JBI Message Exchanges
     */
    protected WorkerPoolManager workerManager;

    /**
     * All the component interceptors. Key is the name of the interceptor
     */
    protected Map<String, Interceptor> interceptors;

    /**
     * The asynchronous message handler
     */
    private AsyncMessageHandler asyncMessageHandler;

    /**
     * Manage the performance notifications, exposed as MBean
     */
    private PerformanceNotifier performanceNotifier;

    /**
     * Manage the interceptors at runtime, exposed as MBean
     */
    private InterceptorManager interceptorManager;

    /**
     * Manage runtime configuration notifications, exposed as MBean
     */
    protected RuntimeConfigurationNotifier runtimeConfigurationNotifier;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getChannel()
     */
    public DeliveryChannel getChannel() {
        return this.channel;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getContext()
     */
    public ComponentContext getContext() {
        return this.context;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getLifeCycle()
     */
    public ComponentLifeCycle getLifeCycle() {
        return this;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getLogger()
     */
    public Logger getLogger() {
        return this.logger;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * javax.jbi.component.Component#getServiceDescription(javax.jbi.servicedesc
     * .ServiceEndpoint)
     */
    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#getServiceDescription(javax
     * .jbi.servicedesc.ServiceEndpoint)
     */
    public Document getServiceDescription(final ServiceEndpoint endpoint) {
        return this.suManager.getServiceDescription(endpoint);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#getServiceUnitListener()
     */
    public ServiceUnitListener getServiceUnitListener() {
        return this.serviceUnitListener;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#
     * getServiceUnitExtensionsValidator()
     */
    public ServiceUnitExtensionsValidator getServiceUnitExtensionsValidator() {
        return this.serviceUnitExtensionsValidator;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getInterceptors()
     */
    public Map<String, Interceptor> getInterceptors() {
        return this.interceptors;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#getInterceptor(java.lang
     * .String)
     */
    public Interceptor getInterceptor(final String name) {
        Interceptor interceptor = null;

        if (this.interceptors != null) {
            interceptor = this.interceptors.get(name);
        }
        return interceptor;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.Component#getServiceUnitManager()
     */
    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getServiceUnitManager()
     */
    public javax.jbi.component.ServiceUnitManager getServiceUnitManager() {
        if (this.suManager == null) {
            this.suManager = this.createServiceUnitManager();
        }
        return this.suManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#getAsyncMessageHandler()
     */
    public AsyncMessageHandler getAsyncMessageHandler() {
        if (this.asyncMessageHandler == null) {
            this.asyncMessageHandler = new AsyncMessageHandler(this.logger);
        }
        return this.asyncMessageHandler;
    }

    public org.ow2.petals.component.framework.jbidescriptor.generated.Component getComponentConfiguration() {
        return this.jbiComponentConfiguration.getComponent();
    }

    public ConfigurationExtensions getComponentExtensions() {
        return this.componentExtensions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.Component#isExchangeWithConsumerOkay
     * (javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithConsumerOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange) {
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.Component#isExchangeWithProviderOkay
     * (javax.jbi.servicedesc.ServiceEndpoint,
     * javax.jbi.messaging.MessageExchange)
     */
    public boolean isExchangeWithProviderOkay(final ServiceEndpoint endpoint,
            final MessageExchange exchange) {
        return true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.api.Component#resolveEndpointReference
     * (org.w3c.dom.DocumentFragment)
     */
    public ServiceEndpoint resolveEndpointReference(final DocumentFragment epr) {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#setServiceUnitListener(org
     * .ow2.petals.component.framework.su.ServiceUnitListener)
     */
    public void setServiceUnitListener(final ServiceUnitListener serviceUnitListener) {
        this.serviceUnitListener = serviceUnitListener;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.Component#
     * setServiceUnitExtensionsValidator
     * (org.ow2.petals.component.framework.su.ServiceUnitExtensionsValidator)
     */
    public void setServiceUnitExtensionsValidator(
            final ServiceUnitExtensionsValidator serviceUnitExtensionsValidator) {
        this.serviceUnitExtensionsValidator = serviceUnitExtensionsValidator;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#init(javax.jbi.component
     * .ComponentContext)
     */
    public final void init(final ComponentContext compContext) throws JBIException {
        this.context = compContext;
        this.logger = this.context.getLogger("", this.getResourceBundleName());

        this.logger.fine("Component initialization in progress...");

        this.channel = this.context.getDeliveryChannel();

        // Get the component configuration
        final File jbiXmlFile = new File(compContext.getInstallRoot() + File.separator + META_INF
                + File.separator + JBI_XML);
        try {
            // CDK configuration
            this.jbiComponentConfiguration = JBIDescriptorBuilder
                    .buildJavaJBIDescriptor(new FileInputStream(jbiXmlFile));

            // Component specific configuration
            if (this.jbiComponentConfiguration.getComponent().getPropertiesFile() != null
                    && !StringHelper.isNullOrEmpty(this.jbiComponentConfiguration.getComponent()
                            .getPropertiesFile().getValue())) {
                this.componentExtensions = new ConfigurationExtensions(
                        this.jbiComponentConfiguration.getComponent().getAny(),
                        this.jbiComponentConfiguration.getComponent().getPropertiesFile()
                                .getValue());
            } else {
                this.componentExtensions = new ConfigurationExtensions(
                        this.jbiComponentConfiguration.getComponent().getAny());
            }
        } catch (final CDKJBIDescriptorException e) {
            throw new JBIException(e);
        } catch (FileNotFoundException e) {
            throw new JBIException(e);
        } catch (PEtALSCDKException e) {
            throw new JBIException(e);
        }

        this.runtimeConfigurationNotifier = new RuntimeConfigurationNotifier(
                this.jbiComponentConfiguration, jbiXmlFile, this.logger);

        this.interceptors = this.loadInterceptors();
        this.interceptorManager = new InterceptorManager(this, this.jbiComponentConfiguration,
                jbiXmlFile);

        // Necessary resources must be instantiated only once during the
        // life-cycle of the component
        if (this.suManager == null) {
            this.suManager = this.createServiceUnitManager();
        }

        this.logger.fine("Calling specific initialization...");
        this.doInit();
        this.logger.fine("Specific initialization done.");

        // Since the worker manager instantiates the JBIListeners, we must
        // initialize it after the component specific initialization. This is
        // because the JBIListener implementation can access to the specific
        // component implementation which is done in the previous this.doInit()
        // call.
        this.workerManager = this.createWorkerManager();
        this.acceptorManager = this.createJBIAcceptorManager();

        this.logger.log(Level.INFO, "Component initialized");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#start()
     */
    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#start()
     */
    public final void start() throws JBIException {
        this.logger.log(Level.INFO, "Start Component");

        if (this.channel == null) {
            this.channel = this.context.getDeliveryChannel();
        }

        this.registerRuntimeConfigMBean();
        this.registerInterceptorManagerMBean();
        this.registerPerformanceMBean();
        this.workerManager.start();
        this.acceptorManager.start();

        this.logger.fine("Calling specific start...");
        this.doStart();
        this.logger.fine("Specific start done.");
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#stop()
     */
    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#stop()
     */
    public final void stop() throws JBIException {
        this.logger.log(Level.INFO, "Stop Component");

        try {
            this.logger.fine("Calling specific stop...");
            this.doStop();
            this.logger.fine("Specific stop done.");
        } finally {
            this.unregisterRuntimeConfigurationMBean();
            this.unregisterInterceptorManagerMBean();
            this.unregisterPerformanceMBean();

            // stop first the acceptor Manager as it aggregates the worker
            // manager
            this.acceptorManager.stop();
            this.workerManager.stop();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.jbi.component.ComponentLifeCycle#shutDown()
     */
    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#shutDown()
     */
    public final void shutDown() throws JBIException {
        this.logger.log(Level.INFO, "Shutdown Component");
        try {
            this.logger.fine("Calling specific shutdown...");
            this.doShutdown();
            this.logger.fine("Specific shutdown done.");
        } finally {
            // remove unnecessary resources
            this.channel = null;
            if (this.asyncMessageHandler != null) {
                this.asyncMessageHandler.shutdown();
                this.asyncMessageHandler = null;
            }
            this.runtimeConfigurationNotifier = null;
            this.performanceNotifier = null;
            this.interceptors = null;
            this.interceptorManager = null;
            this.acceptorManager = null;
            this.workerManager = null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.component.framework.Component#getListenerManager()
     */
    public JBIAcceptorManager getListenerManager() {
        return this.acceptorManager;
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.Component#
     * createPerformanceNotificationReceivingFromOutside(java.lang.String,
     * org.ow2.petals.component.framework.util.Exchange)
     */
    public PerformanceNotification createPerformanceNotificationReceivingFromOutside(
            final String userData, final Exchange msgExchange) throws JBIException {
        return this.createPerformanceNotification(PerformanceNotification.RECEIVING_FROM_OUTSIDE,
                userData, msgExchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.Component#
     * createPerformanceNotificationSendingToOutside(java.lang.String,
     * org.ow2.petals.component.framework.util.Exchange)
     */
    public PerformanceNotification createPerformanceNotificationSendingToOutside(
            final String userData, final Exchange msgExchange) throws JBIException {
        return this.createPerformanceNotification(PerformanceNotification.SENDING_TO_OUTSIDE,
                userData, msgExchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.Component#
     * createPerformanceNotificationReceivingFromNMR(java.lang.String,
     * org.ow2.petals.component.framework.util.Exchange)
     */
    public PerformanceNotification createPerformanceNotificationReceivingFromNMR(
            final String userData, final Exchange msgExchange) throws JBIException {
        return this.createPerformanceNotification(PerformanceNotification.RECEIVING_FROM_NMR,
                userData, msgExchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @seeorg.ow2.petals.component.framework.Component#
     * createPerformanceNotificationSendingToNMR(java.lang.String,
     * org.ow2.petals.component.framework.util.Exchange)
     */
    public PerformanceNotification createPerformanceNotificationSendingToNMR(final String userData,
            final Exchange msgExchange) throws JBIException {
        return this.createPerformanceNotification(PerformanceNotification.SENDING_TO_NMR, userData,
                msgExchange);
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.component.framework.Component#sendPerformanceNotification
     * (org.ow2.petals.component.framework.mbean.PerformanceNotification)
     */
    public void sendPerformanceNotification(final PerformanceNotification performanceNotification) {
        if (this.jbiComponentConfiguration.getComponent().getPerformanceNotifications() != null
                && this.jbiComponentConfiguration.getComponent().getPerformanceNotifications()
                        .isValue()) {
            this.performanceNotifier.send(performanceNotification);
        }
    }

    /**
     * Can be overridden to return a custom resource bundle name
     * 
     * @return the resource bundle name or null if no resource bundle name has
     *  to be used (default value)
     */
    protected String getResourceBundleName() {
        return null;
    }

    /**
     * Can be overridden to provide a specific service unit manager
     * 
     * @return the service unit manager used by this component.
     */
    protected abstract ServiceUnitManager createServiceUnitManager();

    /**
     * Creates a new {@link JBIAcceptorManager}
     * 
     * @return the created {@link JBIAcceptorManager}
     */
    protected JBIAcceptorManager createJBIAcceptorManager() {
        return new JBIAcceptorManager(this, this.workerManager, this.runtimeConfigurationNotifier,
                this.logger);
    }

    /**
     * Create the worker manager
     * 
     * @return
     */
    protected WorkerPoolManager createWorkerManager() {
        return new JBIWorkerManager(this, this.runtimeConfigurationNotifier, this.logger);
    }

    /**
     * A convenience method which can be overridden so that there's no need to
     * call super.doInit(). This method is called at the end of the component
     * classical initialization. When this method is called, the component's
     * delivery channel, context, logger and service unit manager are available.
     * 
     */
    protected void doInit() throws JBIException {
    }

    /**
     * A convenience method which can be overridden so that there's no need to
     * call super.doStart(). This method is called at the end of the component
     * classical start.
     * 
     */
    protected void doStart() throws JBIException {
    }

    /**
     * A convenience method which can be overridden so that there's no need to
     * call super.doStop(). This method is called at the end of the component
     * classical stop.
     * 
     */
    protected void doStop() throws JBIException {
    }

    /**
     * A convenience method which can be overridden so that there's no need to
     * call super.doShutdown(). This method is called at the end of the
     * component classical shutdown.
     * 
     */
    protected void doShutdown() throws JBIException {
    }

    /**
     * Load the interceptors if there is any.
     * 
     * @param configuration
     */
    private final Map<String, Interceptor> loadInterceptors() {
        final Map<String, Interceptor> interceptors = new HashMap<String, Interceptor>();

        if (this.jbiComponentConfiguration.getComponent().getComponentInterceptors() != null) {
            for (final ComponentInterceptor componentInterceptor : this.jbiComponentConfiguration
                    .getComponent().getComponentInterceptors().getInterceptor()) {
                try {
                    this.logger.info("Loading interceptor : " + componentInterceptor.getName());
                    final Interceptor itc = InterceptorBuilder.loadInstance(componentInterceptor
                            .getClazz());

                    ConfigurationExtensions configurationExtensions;
                    if (this.jbiComponentConfiguration.getComponent().getPropertiesFile() != null
                            && !StringHelper.isNullOrEmpty(this.jbiComponentConfiguration
                                    .getComponent().getPropertiesFile().getValue())) {
                        configurationExtensions = new ConfigurationExtensions(componentInterceptor
                                .getParam(), this.jbiComponentConfiguration.getComponent()
                                .getPropertiesFile().getValue());
                    } else {
                        configurationExtensions = new ConfigurationExtensions(componentInterceptor
                                .getParam());
                    }

                    itc.init(this, componentInterceptor.getName(), componentInterceptor.isActive(),
                            configurationExtensions, this.logger);
                    interceptors.put(componentInterceptor.getName(), itc);
                } catch (final PEtALSCDKException e) {
                    this.logger.log(Level.WARNING, "Could not load or initialize interceptor '"
                            + componentInterceptor.getName() + "' from class '"
                            + componentInterceptor.getClazz() + "', this interceptor is skipped");
                }
            }
        }

        return interceptors;
    }

    /**
     * Create a new notification
     * 
     * @param type
     * @param userData
     * @param msgExchange
     * @return
     * @throws JBIException
     */
    private PerformanceNotification createPerformanceNotification(final String type,
            final String userData, final Exchange msgExchange) throws JBIException {
        PerformanceNotification performanceNotification = null;

        final Runtimeboolean performanceNotifEnabled = this.jbiComponentConfiguration
                .getComponent().getPerformanceNotifications();
        if (performanceNotifEnabled != null && performanceNotifEnabled.isValue()) {
            // If the performance notification is enabled during runtime, the
            // performance notifier MBean need to be registered
            if (this.performanceNotifier == null) {
                registerPerformanceMBean();

            }
            performanceNotification = this.performanceNotifier.createPerformanceNotifications(
                    System.currentTimeMillis(), this.context.getComponentName(), type, userData,
                    msgExchange);
        }
        return performanceNotification;
    }

    /**
     * Register the {@link RuntimeConfigurationMBean}
     * 
     * @throws JBIException
     */
    private void registerRuntimeConfigMBean() throws JBIException {
        final ObjectName runtimeConfigurationMBeanName = this.context.getMBeanNames()
                .createCustomComponentMBeanName(
                        "runtime_configuration_" + this.context.getComponentName());

        this.runtimeConfigurationNotifier.setMBeanName(runtimeConfigurationMBeanName);

        MBeanHelper.registerMBean(this.context.getMBeanServer(), runtimeConfigurationMBeanName,
                this.runtimeConfigurationNotifier);
    }

    /**
     * 
     * @throws JBIException
     */
    private void registerInterceptorManagerMBean() throws JBIException {
        final ObjectName mbeanName = this.context.getMBeanNames().createCustomComponentMBeanName(
                "interceptors_" + this.context.getComponentName());

        this.interceptorManager.setMBeanName(mbeanName);

        MBeanHelper
                .registerMBean(this.context.getMBeanServer(), mbeanName, this.interceptorManager);
    }

    /**
     * @throws JBIException
     */
    private void registerPerformanceMBean() throws JBIException {
        if (this.jbiComponentConfiguration.getComponent().getPerformanceNotifications() != null
                && this.jbiComponentConfiguration.getComponent().getPerformanceNotifications()
                        .isValue()) {
            final ObjectName mbeanName = this.context.getMBeanNames()
                    .createCustomComponentMBeanName(
                            "performance_" + this.context.getComponentName());

            this.performanceNotifier = new PerformanceNotifier(this.getComponentConfiguration(),
                    mbeanName);

            MBeanHelper.registerMBean(this.context.getMBeanServer(), mbeanName,
                    this.performanceNotifier);
        }
    }

    /**
     * Unregister the runtime configuration MBean
     * 
     * @throws JBIException
     */
    private void unregisterRuntimeConfigurationMBean() throws JBIException {
        MBeanHelper.unregisterMBean(this.context.getMBeanServer(),
                this.runtimeConfigurationNotifier.getMBeanName());
    }

    /**
     * 
     * @throws JBIException
     */
    private void unregisterInterceptorManagerMBean() throws JBIException {
        MBeanHelper.unregisterMBean(this.context.getMBeanServer(), this.interceptorManager
                .getMBeanName());
    }

    /**
     * Unregister the runtime configuration MBean
     * 
     * @throws JBIException
     */
    private void unregisterPerformanceMBean() throws JBIException {
        if (this.performanceNotifier != null) {
            MBeanHelper.unregisterMBean(this.context.getMBeanServer(), this.performanceNotifier
                    .getMBeanName());
        }
    }

}
